@@ -1,7 +1,8 @@
 # -*- coding: utf-8 -*-
 #
 # This file is part of Flask-Menu
-# Copyright (C) 2013, 2014 CERN.
+# Copyright (C) 2013, 2014, 2015, 2017 CERN
+# Copyright (C) 2017 Marlin Forbes
 #
 # Flask-Menu is free software; you can redistribute it and/or modify
 # it under the terms of the Revised BSD License; see LICENSE file for
@@ -12,16 +13,27 @@
 Those menus can be then displayed using templates.
 """
 
-from flask import url_for, current_app, request, Blueprint
+import inspect
+import types
+
+from flask import Blueprint, current_app, g, request, url_for
+from six import PY3
 from werkzeug.local import LocalProxy
+
 from .version import __version__
 
-CONDITION_TRUE = lambda: True
-CONDITION_FALSE = lambda: False
+
+def CONDITION_TRUE(*args, **kwargs):
+    """Return always True."""
+    return True
+
+
+def CONDITION_FALSE(*args, **kwargs):
+    """Return always False."""
+    return False
 
 
 class Menu(object):
-
     """Flask extension implementation."""
 
     def __init__(self, app=None):
@@ -42,6 +54,12 @@ class Menu(object):
         app.context_processor(lambda: dict(
             current_menu=current_menu))
 
+        @app.url_value_preprocessor
+        def url_preprocessor(route, args):
+            """Store the current route endpoint and arguments."""
+            g._menu_kwargs = args
+            g._menu_route = route
+
     @staticmethod
     def root():
         """Return a root entry of current application's menu."""
@@ -49,7 +67,6 @@ class Menu(object):
 
 
 class MenuEntryMixin(object):
-
     """Represent a entry node in the menu tree.
 
     Provides information for displaying links (text, url, visible, active).
@@ -64,25 +81,62 @@ class MenuEntryMixin(object):
         self._endpoint = None
         self._text = None
         self._order = 0
+        self._external_url = None
         self._endpoint_arguments_constructor = None
         self._dynamic_list_constructor = None
-        self._active_when = lambda: request.endpoint == self._endpoint
         self._visible_when = CONDITION_TRUE
+        self._expected_args = []
 
-    def register(self, endpoint, text, order=0,
+    def _active_when(self):
+        """Define condition when a menu entry is active."""
+        matching_endpoint = request.endpoint == self._endpoint
+
+        def segments(path):
+            """Split a path into segments."""
+            parts = path.split('/')[1:]
+            if len(parts) > 0 and parts[-1] == '':
+                parts.pop()
+            return parts
+
+        if len(self.url) > 1:
+            segments_url = segments(self.url)
+            segments_request = segments(request.path)
+            matching_segments = \
+                segments_request[0:len(segments_url)] == segments_url
+        else:
+            matching_segments = False
+        matching_completpath = request.path == self.url
+        return matching_endpoint or matching_segments or matching_completpath
+
+    def register(self, endpoint=None, text=None, order=0, external_url=None,
                  endpoint_arguments_constructor=None,
                  dynamic_list_constructor=None,
                  active_when=None,
                  visible_when=None,
+                 expected_args=None,
                  **kwargs):
-        """Assign endpoint and display values."""
+        """Assign endpoint and display values.
+
+        .. versionadded:: 0.6.0
+           The *external_url* parameter is mutually exclusive with *endpoint*.
+        """
+        if endpoint is not None and external_url is not None:
+            raise TypeError('Exclusive arguments endpoint and external_url.')
+
         self._endpoint = endpoint
-        self._text = text
+        self._text = text or self.name
         self._order = order
+        self._external_url = external_url
+        self._expected_args = expected_args or []
         self._endpoint_arguments_constructor = endpoint_arguments_constructor
         self._dynamic_list_constructor = dynamic_list_constructor
         if active_when is not None:
-            self._active_when = active_when
+            active_when_param = inspect.getfullargspec(active_when)[0] \
+                if PY3 else inspect.getargspec(active_when)[0]
+            if len(active_when_param) == 1:
+                self._active_when = types.MethodType(active_when, self)
+            else:
+                self._active_when = active_when
         if visible_when is not None:
             self._visible_when = visible_when
 
@@ -164,6 +218,24 @@ class MenuEntryMixin(object):
         """Make the entry always hidden."""
         self._visible_when = CONDITION_FALSE
 
+    @property
+    def active_item(self):
+        """Return the active item from the menu's tree.
+
+        Return self if the item itself is active. Return an
+        active child if there is one. If there are no active menu items,
+        None will be returned.
+        """
+        for child in self.children:
+            active = child.active_item
+            if active is not None:
+                return active
+
+        if self.active:
+            return self
+
+        return None
+
     @property
     def dynamic_list(self):
         """Return list from dynamic list constructor."""
@@ -190,12 +262,22 @@ class MenuEntryMixin(object):
     def url(self):
         """Generate url from given endpoint and optional dynamic arguments."""
         if not self._endpoint:
-            return '#'
+            return self._external_url or '#'
 
         if self._endpoint_arguments_constructor:
             return url_for(self._endpoint,
                            **self._endpoint_arguments_constructor())
-        return url_for(self._endpoint)
+
+        # Inject current args. Allows usage when inside a blueprint with a url
+        # param.
+        # Filter out any arguments which don't need to be passed.
+        args = {}
+        if hasattr(g, '_menu_kwargs') and g._menu_kwargs:
+            for key in g._menu_kwargs:
+                if key in self._expected_args:
+                    args[key] = g._menu_kwargs[key]
+
+        return url_for(self._endpoint, **args)
 
     @property
     def active(self):
@@ -207,6 +289,31 @@ class MenuEntryMixin(object):
         """Return True if the menu item is visible."""
         return self._text is not None and self._visible_when()
 
+    def has_active_child(self, recursive=True):
+        """Return True if the menu has an active child."""
+        for child in self._child_entries.values():
+            if child.active:
+                return True
+        if recursive:
+            for child in self._child_entries.values():
+                if child.has_active_child(recursive=recursive):
+                    return True
+        return False
+
+    def has_visible_child(self, recursive=True):
+        """Return True if the menu has a visible child."""
+        result = False
+        for child in self._child_entries.values():
+            if child.visible:
+                return True
+
+        if recursive:
+            for child in self._child_entries.values():
+                if child.visible or child.has_visible_child(recursive=True):
+                    return True
+
+        return False
+
 
 def register_menu(app, path, text, order=0,
                   endpoint_arguments_constructor=None,
@@ -256,6 +363,9 @@ def register_menu(app, path, text, order=0,
             endpoint = f.__name__
             before_first_request = app.before_first_request
 
+        expected = inspect.getfullargspec(f).args if PY3 else \
+            inspect.getargspec(f).args
+
         @before_first_request
         def _register_menu_item():
             # str(path) allows path to be a string-convertible object
@@ -269,6 +379,7 @@ def register_menu(app, path, text, order=0,
                 dynamic_list_constructor=dynamic_list_constructor,
                 active_when=active_when,
                 visible_when=visible_when,
+                expected_args=expected,
                 **kwargs)
         return f
 
